@@ -3,7 +3,7 @@
 ##  utils.R                                                                  ##
 ##  Part of the geomtextpath R package                                       ##
 ##                                                                           ##
-##  Copyright (C) 2021 by Allan Cameron & Teun van den Brand                 ##
+##  Copyright (C) 2021 - 2022 by Allan Cameron & Teun van den Brand          ##
 ##                                                                           ##
 ##  Licensed under the MIT license - see https://mit-license.org             ##
 ##  or the LICENSE file in the project root directory                        ##
@@ -16,6 +16,7 @@
 # wrapper simply helps keep the code more legible / maintainable.
 
 numapply <- function(data, fun) {
+
   vapply(data, FUN = fun, FUN.VALUE = numeric(1))
 }
 
@@ -24,6 +25,7 @@ numapply <- function(data, fun) {
 # of several data frames in a list.
 
 nrow_multi <- function(data) {
+
   vapply(data, FUN = nrow, FUN.VALUE = integer(1), USE.NAMES = FALSE)
 }
 
@@ -33,6 +35,7 @@ nrow_multi <- function(data) {
 # Gives the indices of a vector where a run length starts
 
 run_start <- function(x) {
+
   x <- discretise(x)
   c(0, which(diff(x) != 0)) + 1
 }
@@ -41,6 +44,7 @@ run_start <- function(x) {
 # Simplified rle(x)$lengths
 
 run_len <- function(x) {
+
   diff(c(run_start(x), length(x) + 1))
 }
 
@@ -51,6 +55,7 @@ run_len <- function(x) {
 # Only use when `...` has valid names and content are valid data.frame columns
 
 data_frame <- function(...) {
+
   list_to_df(x = list(...))
 }
 
@@ -60,6 +65,7 @@ data_frame <- function(...) {
 # without bothering with 'n'/'size'
 
 list_to_df <- function(x = list()) {
+
   if (length(x) != 0 && is.null(names(x))) {
     stop("Elements must be named", call. = FALSE)
   }
@@ -86,6 +92,7 @@ list_to_df <- function(x = list()) {
 # `idcol` is a name for an id column, or NULL if it is to be omitted
 
 rbind_dfs <- function(df_list) {
+
   do.call(rbind.data.frame, c(df_list, make.row.names = FALSE))
 }
 
@@ -95,6 +102,7 @@ rbind_dfs <- function(df_list) {
 #  Convert unique values to ordered sequence of integers
 
 discretise <- function(x) {
+
   match(x, unique(x))
 }
 
@@ -111,6 +119,7 @@ group_id <- function(data, vars) {
 # Shorthand for `vapply(split(x, group), ...)`
 
 gapply <- function(x, group, FUN, FUN.VALUE, ..., USE.NAMES = FALSE) {
+
   vapply(
     split(x, group),
     FUN = FUN, FUN.VALUE = FUN.VALUE,
@@ -119,6 +128,7 @@ gapply <- function(x, group, FUN, FUN.VALUE, ..., USE.NAMES = FALSE) {
   )
 }
 
+
 # Approx utilities -------------------------------------------------------------
 
 # Function for `approx()`-ing a number of `y` variables. More efficient
@@ -127,6 +137,7 @@ gapply <- function(x, group, FUN, FUN.VALUE, ..., USE.NAMES = FALSE) {
 # Note: This also extrapolates based on the four extreme points.
 
 approx_multi <- function(x, y = matrix(), xout) {
+
   if (length(y) == 0) return(y)
 
   # Coerce lists and data.frames to matrices
@@ -177,6 +188,7 @@ approx_multi <- function(x, y = matrix(), xout) {
 # Simple linear interpolation for NA values
 
 interp_na <- function(x) {
+
   if (!anyNA(x)) return(x)
   stopifnot("Cannot interpolate NA in non-numeric vectors" = is.numeric(x))
   x[] <- approx(seq_along(x), x, seq_along(x))$y
@@ -187,6 +199,7 @@ interp_na <- function(x) {
 # Re-implementation of unexported ggplot function
 
 cases <- function(x, fun) {
+
   ok <- vapply(x, fun, logical(nrow(x)))
   if (is.vector(ok)) {
     all(ok)
@@ -199,6 +212,7 @@ cases <- function(x, fun) {
 # Consistent missingness check for different input types
 
 is_missing <- function(x) {
+
   if (typeof(x) == "list") !vapply(x, is.null, logical(1)) else !is.na(x)
 }
 
@@ -206,6 +220,7 @@ is_missing <- function(x) {
 # Consistent infinity check for different input types
 
 is_finite <- function(x) {
+
   if (typeof(x) == "list") !vapply(x, is.null, logical(1)) else is.finite(x)
 }
 
@@ -213,6 +228,7 @@ is_finite <- function(x) {
 # Finds rows with missing data in given data frame
 
 detect_missing <- function(df, vars, finite = FALSE) {
+
     vars <- intersect(vars, names(df))
     !cases(df[, vars, drop = FALSE], if (finite) is_finite else is_missing)
 }
@@ -221,6 +237,7 @@ detect_missing <- function(df, vars, finite = FALSE) {
 # Finds missing aesthetics in input data frame
 
 find_missing <- function(x, layer) {
+
   detect_missing(x, c(layer$required_aes, layer$non_missing_aes))
 }
 
@@ -230,6 +247,7 @@ find_missing <- function(x, layer) {
 # S3 generic to ensure label vector matches to given object
 
 match_labels <- function(x, ...) {
+
   UseMethod("match_labels")
 }
 
@@ -237,6 +255,7 @@ match_labels <- function(x, ...) {
 #S3 method to ensure labels are same length as data frame
 
 match_labels.data.frame <- function(x, labels) {
+
   if (length(labels) == 1) labels <- rep(labels, nrow(x))
   if (nrow(x) != length(labels)) {
     stop("Could not match labels to object ", deparse(substitute(x)))
@@ -248,6 +267,7 @@ match_labels.data.frame <- function(x, labels) {
 # match labels to vector or list
 
 match_labels.default <- function(x, labels) {
+
   if (length(labels) == 1) labels <- rep(labels, length(x))
   if (length(x) != length(labels)) {
     stop("Could not match labels to object ", deparse(substitute(x)))
@@ -259,6 +279,7 @@ match_labels.default <- function(x, labels) {
 # Similar to ggplot's rename function
 
 rename <- function(x, replace) {
+
   current_names <- names(x)
   old_names     <- names(replace)
   missing_names <- setdiff(old_names, current_names)
@@ -277,10 +298,11 @@ rename <- function(x, replace) {
 # Parse characters as expressions with validity checks
 
 safe_parse <- function(text) {
+
   if (!is.character(text)) stop("`text` must be a character vector")
   out <- vector("expression", length(text))
   for (i in seq_along(text)) {
-      expr <- parse(text = text[[i]])
+      expr     <- parse(text = text[[i]])
       out[[i]] <- if (length(expr) == 0) NA else expr[[1]]
   }
   out
@@ -291,6 +313,7 @@ safe_parse <- function(text) {
 # glyphs or as a single unit
 
 is.multichar <- function(x) {
+
   if (is.list(x)) return(any(vapply(x, is.multichar, logical(1))))
   if (is.factor(x)) x <- as.character(x)
   if (is.character(x)) return(any(nchar(x) > 1))
@@ -301,6 +324,7 @@ is.multichar <- function(x) {
 # Based on ggplot2:::draw_axis handling of labels
 
 make_label <- function(x) {
+
   if (!is.list(x)) return(x)
   if (any(vapply(x, is.language, logical(1)))) {
     do.call(expression, x)
@@ -313,6 +337,7 @@ make_label <- function(x) {
 # Allows information about co-ordinate system to be passed to grob
 
 get_polar_params <- function(coord) {
+
   if (inherits(coord, "CoordPolar")) {
       list(x = 0.5, y = 0.5, theta = coord$theta)
     } else  {
@@ -326,6 +351,7 @@ get_polar_params <- function(coord) {
 # Convert internal ggplot-type aesthetic data frame to gp object for text
 
 data_to_text_gp <- function(data) {
+
   gpar(
     col        = alpha(data$textcolour %||% data$colour, data$alpha),
     fontsize   = data$size * .pt,
@@ -339,8 +365,12 @@ data_to_text_gp <- function(data) {
 
 # Convert internal ggplot-type aesthetic data frame to gp object for lines
 
-data_to_path_gp <- function(data, lineend = "butt", linejoin = "round",
-                            linemitre = 10) {
+data_to_path_gp <- function(
+  data,
+  lineend   = "butt",
+  linejoin  = "round",
+  linemitre = 10) {
+
   if (all(data$linetype %in% c("0", "blank", NA))) {
     gpar(lty = 0)
   } else {
@@ -359,8 +389,12 @@ data_to_path_gp <- function(data, lineend = "butt", linejoin = "round",
 
 # Convert internal ggplot-type aesthetic data frame to gp object for lines
 
-data_to_box_gp <- function(data, lineend = "butt", linejoin = "round",
-                           linemitre = 10) {
+data_to_box_gp <- function(
+  data,
+  lineend   = "butt",
+  linejoin  = "round",
+  linemitre = 10) {
+
   gpar(
     col       = alpha(data$boxcolour %||% data$linecolour %||% data$colour,
                       data$alpha),
@@ -377,6 +411,7 @@ data_to_box_gp <- function(data, lineend = "butt", linejoin = "round",
 # Helper function to do safe(r) recycling on "gpar" class objects.
 
 recycle_gp <- function(gp, fun, ...) {
+
   do_recycle     <- lengths(gp) > 1
   gp[do_recycle] <- lapply(unclass(gp)[do_recycle], fun, ...)
   gp[lengths(gp) == 0] <- list(NULL)
@@ -387,6 +422,7 @@ recycle_gp <- function(gp, fun, ...) {
 # Safely repeat values inside a gpar object
 
 rep_gp <- function(gp, length.out = max(lengths(gp))) {
+
   gp[] <- lapply(gp, rep, length.out = length.out)
   gp
 }
@@ -395,6 +431,7 @@ rep_gp <- function(gp, length.out = max(lengths(gp))) {
 # Split gpar into llist of ength-1 gpars
 
 split_gp <- function(gp, i = seq_len(max(lengths(gp)))) {
+
   gp <- rep_gp(gp, max(i))
   lapply(i, function(j) recycle_gp(gp, `[`, j))
 }
@@ -404,6 +441,7 @@ split_gp <- function(gp, i = seq_len(max(lengths(gp)))) {
 # Based on ggplot2:::modify_list
 
 gp_fill_defaults <- function(gp, ..., defaults = get.gpar()) {
+
   extra <- list(...)
   for (i in names(extra)) defaults[[i]] <- extra[[i]]
   for (i in names(gp))    defaults[[i]] <- gp[[i]]
@@ -414,6 +452,7 @@ gp_fill_defaults <- function(gp, ..., defaults = get.gpar()) {
 # Safely subset gpar
 
 gp_subset <- function(gp, ss) {
+
   subset_these         <- lengths(gp) > 1
   gp[subset_these]     <- lapply(unclass(gp)[subset_these], function(x) x[ss])
   gp[lengths(gp) == 0] <- list(NULL)
@@ -424,21 +463,25 @@ gp_subset <- function(gp, ss) {
 # Convert grid unit object to given unit type, then make numeric
 
 as_grid_unit <- function(value, from = "x", unit = "native") {
+
   if (!is.unit(value)) return(value)
 
   axis <- list(x = "x", y = "y", width = "x", height = "y")
   type <- list(x = "location", y = "location",
                width = "dimension", height = "dimension")
 
-  convertUnit(x = value, unitTo = unit,
-              axisFrom = axis[[from]], typeFrom = type[[from]],
+  convertUnit(x         = value,
+              unitTo    = unit,
+              axisFrom  = axis[[from]],
+              typeFrom  = type[[from]],
               valueOnly = TRUE)
 }
 
 
-# Convert grid units to number of inched
+# Convert grid units to number of inches
 
 as_inch <- function(x, from = "x") {
+
   as_grid_unit(x, from, "inch")
 }
 
@@ -446,6 +489,7 @@ as_inch <- function(x, from = "x") {
 # Convert grid units to numeric npc values
 
 as_npc <- function(x, from = "x") {
+
   as_grid_unit(x, from, "npc")
 }
 
@@ -453,6 +497,7 @@ as_npc <- function(x, from = "x") {
 # Same as grid::unit, but allows unit objects to safely pass through
 
 as_unit <- function(x, units = NULL, ...) {
+
   if (!is.unit(x) && !is.null(units)) {
     x <- unit(x, units, ...)
   }
@@ -465,6 +510,7 @@ as_unit <- function(x, units = NULL, ...) {
 # Simple lowercase converter
 
 as_lower <- function(x) {
+
   chartr("ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz", x)
 }
 
@@ -472,6 +518,7 @@ as_lower <- function(x) {
 # Simple uppercase converter
 
 as_upper <- function(x) {
+
   chartr("abcdefghijklmnopqrstuvwxyz", "ABCDEFGHIJKLMNOPQRSTUVWXYZ", x)
 }
 
@@ -479,38 +526,44 @@ as_upper <- function(x) {
 # Convert snake_case to CamelCase
 
 camelize <- function(x, first = FALSE) {
-    x <- gsub("_(.)", "\\U\\1", x, perl = TRUE)
-    if (first) {
-        x <- paste0(as_upper(substring(x, 1, 1)), substring(x, 2))
-    }
-    x
+
+  x <- gsub("_(.)", "\\U\\1", x, perl = TRUE)
+  if (first) {
+      x <- paste0(as_upper(substring(x, 1, 1)), substring(x, 2))
+  }
+  x
 }
 
 
 # Find object in given environment with fallback to package namespace
 
 find_global <- function(name, env, mode = "any") {
-    if (exists(name, envir = env, mode = mode)) {
-        return(get(name, envir = env, mode = mode))
-    }
-    nsenv <- asNamespace("geomtextpath")
-    if (exists(name, envir = nsenv, mode = mode)) {
-        return(get(name, envir = nsenv, mode = mode))
-    }
-    NULL
+
+  if (exists(name, envir = env, mode = mode)) {
+      return(get(name, envir = env, mode = mode))
+  }
+  nsenv <- asNamespace("geomtextpath")
+  if (exists(name, envir = nsenv, mode = mode)) {
+      return(get(name, envir = nsenv, mode = mode))
+  }
+  NULL
 }
 
 # Finds object of given type and class (as characters if necessary)
 
-check_subclass <- function(x, subclass, argname = as_lower(subclass),
-                           env = parent.frame()) {
+check_subclass <- function(
+  x,
+  subclass,
+  argname = as_lower(subclass),
+  env     = parent.frame()
+  ) {
+
   if (inherits(x, subclass)) {
     x
   }
   else if (is.character(x) && length(x) == 1) {
-
     name <- paste0(subclass, camelize(x, first = TRUE))
-    obj <- find_global(name, env = env)
+    obj  <- find_global(name, env = env)
     if (is.null(obj) || !inherits(obj, subclass)) {
         abort(paste0("Can't find `", argname, "` called '", x, "'"))
     } else {
@@ -523,78 +576,82 @@ check_subclass <- function(x, subclass, argname = as_lower(subclass),
   }
 }
 
+
 # Documentation functions adapted from ggplot2 ---------------------------------
 
 # Adds each aesthetic to output in rd_aesthetics
 
 rd_aesthetics_item <- function(x) {
-    req <- x$required_aes
-    req <- sub("|", "} \\emph{or} \\code{", req,
-        fixed = TRUE)
-    req_aes <- unlist(strsplit(x$required_aes, "|", fixed = TRUE))
-    optional_aes <- setdiff(x$aesthetics(), req_aes)
-    all <- union(req, sort(optional_aes))
-    ifelse(all %in% req, paste0("\\strong{\\code{", all,
-        "}}"), paste0("\\code{", all, "}"))
+
+  req          <- x$required_aes
+  req          <- sub("|", "} \\emph{or} \\code{", req, fixed = TRUE)
+  req_aes      <- unlist(strsplit(x$required_aes, "|", fixed = TRUE))
+  optional_aes <- setdiff(x$aesthetics(), req_aes)
+  all          <- union(req, sort(optional_aes))
+
+  ifelse(test = all %in% req,
+         yes  = paste0("\\strong{\\code{", all, "}}"),
+         no   = paste0("\\code{", all, "}"))
 }
 
 
 # Calculate aesthetics and produce documentation item for given function / topic
 
 rd_aesthetics <- function(type, name, check_label_variant = TRUE) {
-    obj <- switch(type,
-                  geom = check_subclass(name, "Geom", env = globalenv()),
-                  stat = check_subclass(name, "Stat", env = globalenv()))
-    aes <- rd_aesthetics_item(obj)
-    txt <- "@section Aesthetics:"
-    txt <- c(txt,
-             paste0("\\code{", type,
-                    "_", name, "()} ",
-                    "understands the following aesthetics ",
-                    "(required aesthetics are in bold):"))
-    txt <- c(txt, "\\itemize{", paste0("  \\item ", aes), "}")
 
-    lab_aes <- NULL
-    if (check_label_variant) {
-      # Check if there is 'text' to be substituted by 'label'
-      lab_name <- gsub("^text", "label", name)
-      if (!(lab_name == name)) {
-        # Check if label variant exists
-        lab_obj <- tryCatch(
-          {
-            switch(type,
-                   geom = check_subclass(lab_name, "Geom", env = globalenv()),
-                   stat = check_subclass(lab_name, "Stat", env = globalenv()))
-          },
-          error   = function(cond) {return(NULL)}
-        )
-        if (!is.null(lab_obj)) {
-          lab_aes <- rd_aesthetics_item(lab_obj)
-          lab_aes <- setdiff(lab_aes, aes)
-        }
-        if (length(lab_aes)) {
-          txt <- c(txt,
-                   paste0("In addition to aforementioned aesthetics,",
-                          " \\code{", type, "_", lab_name, "()} ",
-                          "also understands:"))
-          txt <- c(txt, "\\itemize{", paste0("  \\item ", lab_aes), "}")
-        }
+  obj <- switch(type,
+                geom = check_subclass(name, "Geom", env = globalenv()),
+                stat = check_subclass(name, "Stat", env = globalenv()))
+  aes <- rd_aesthetics_item(obj)
+  txt <- "@section Aesthetics:"
+  txt <- c(txt,
+           paste0("\\code{", type,
+                  "_", name, "()} ",
+                  "understands the following aesthetics ",
+                  "(required aesthetics are in bold):"))
+  txt <- c(txt, "\\itemize{", paste0("  \\item ", aes), "}")
+
+  lab_aes <- NULL
+  if (check_label_variant) {
+    # Check if there is 'text' to be substituted by 'label'
+    lab_name <- gsub("^text", "label", name)
+    if (!(lab_name == name)) {
+      # Check if label variant exists
+      lab_obj <- tryCatch(
+        {
+          switch(type,
+                 geom = check_subclass(lab_name, "Geom", env = globalenv()),
+                 stat = check_subclass(lab_name, "Stat", env = globalenv()))
+        },
+        error   = function(cond) {return(NULL)}
+      )
+      if (!is.null(lab_obj)) {
+        lab_aes <- rd_aesthetics_item(lab_obj)
+        lab_aes <- setdiff(lab_aes, aes)
+      }
+      if (length(lab_aes)) {
+        txt <- c(txt,
+                 paste0("In addition to aforementioned aesthetics,",
+                        " \\code{", type, "_", lab_name, "()} ",
+                        "also understands:"))
+        txt <- c(txt, "\\itemize{", paste0("  \\item ", lab_aes), "}")
       }
     }
+  }
 
-    if (any(grepl("spacing", aes))) {
-      txt <- c(txt,
-               "The \\code{spacing} aesthetic allows fine control of spacing",
-               " of text, which is called 'tracking' in typography.",
-               "The default is 0 and units are measured in 1/1000 em.",
-               "Numbers greater than zero increase the spacing,",
-               "whereas negative numbers decrease the spacing.")
-    }
-
+  if (any(grepl("spacing", aes))) {
     txt <- c(txt,
-             "\n\nLearn more about setting these aesthetics ",
-             "in \\code{vignette(\"ggplot2-specs\")}.")
-    txt
+             "The \\code{spacing} aesthetic allows fine control of spacing",
+             " of text, which is called 'tracking' in typography.",
+             "The default is 0 and units are measured in 1/1000 em.",
+             "Numbers greater than zero increase the spacing,",
+             "whereas negative numbers decrease the spacing.")
+  }
+
+  txt <- c(txt,
+           "\n\nLearn more about setting these aesthetics ",
+           "in \\code{vignette(\"ggplot2-specs\")}.")
+  txt
 }
 
 
@@ -606,6 +663,7 @@ rd_aesthetics <- function(type, name, check_label_variant = TRUE) {
 # to some geom or that kind of situation.
 
 rd_dots <- function(fun, exclude = character()) {
+
   exclude <- c(exclude, ".type")
   params  <- names(formals(static_text_params))
   forms   <- names(formals(fun))
@@ -662,30 +720,31 @@ rd_dots <- function(fun, exclude = character()) {
 # Required for reference line geoms as parameters can overwrite aesthetics
 
 warn_overwritten_args <- function (
-    fun_name,
-    overwritten_arg,
-    provided_args,
-    plural_join = " and/or ")
-{
-    overwritten_arg_text <- paste0("`", overwritten_arg, "`")
-    n_provided_args      <- length(provided_args)
-    if (n_provided_args == 1) {
-        provided_arg_text <- paste0("`", provided_args,
-            "`")
-        verb <- "was"
-    }
-    else if (n_provided_args == 2) {
-        provided_arg_text <- paste0("`", provided_args,
-            "`", collapse = plural_join)
-        verb <- "were"
-    }
-    else {
-        provided_arg_text <- paste0(paste0("`", provided_args[-n_provided_args],
-            "`", collapse = ", "), ",", plural_join,
-            "`", provided_args[n_provided_args], "`")
-        verb <- "were"
-    }
-    warn(paste0(fun_name,
-                ": Ignoring ", overwritten_arg_text, " because ",
-                provided_arg_text, " ", verb, " provided."))
+  fun_name,
+  overwritten_arg,
+  provided_args,
+  plural_join = " and/or "
+  ) {
+
+  overwritten_arg_text <- paste0("`", overwritten_arg, "`")
+  n_provided_args      <- length(provided_args)
+  if (n_provided_args == 1) {
+      provided_arg_text <- paste0("`", provided_args,
+          "`")
+      verb <- "was"
+  }
+  else if (n_provided_args == 2) {
+      provided_arg_text <- paste0("`", provided_args,
+          "`", collapse = plural_join)
+      verb <- "were"
+  }
+  else {
+      provided_arg_text <- paste0(paste0("`", provided_args[-n_provided_args],
+          "`", collapse = ", "), ",", plural_join,
+          "`", provided_args[n_provided_args], "`")
+      verb <- "were"
+  }
+  warn(paste0(fun_name,
+              ": Ignoring ", overwritten_arg_text, " because ",
+              provided_arg_text, " ", verb, " provided."))
 }
